@@ -2,7 +2,7 @@
 This submodule contains the client class that provides most of the SDK functionality.
 """
 
-from typing import Optional, Any, Dict, Mapping
+from typing import Optional, Any, Dict, Mapping, Union
 
 from .impl import AnyNum
 
@@ -11,27 +11,28 @@ import hmac
 import threading
 import traceback
 
-from ldclient.config import Config, HTTPConfig
-from ldclient.diagnostics import create_diagnostic_id, _DiagnosticAccumulator
-from ldclient.event_processor import DefaultEventProcessor
-from ldclient.feature_requester import FeatureRequesterImpl
+from ldclient.config import Config
+from ldclient.context import Context
 from ldclient.feature_store import _FeatureStoreDataSetSorter
 from ldclient.evaluation import EvaluationDetail, FeatureFlagsState
 from ldclient.impl.big_segments import BigSegmentStoreManager
+from ldclient.impl.datasource.feature_requester import FeatureRequesterImpl
+from ldclient.impl.datasource.polling import PollingUpdateProcessor
+from ldclient.impl.datasource.streaming import StreamingUpdateProcessor
 from ldclient.impl.evaluator import Evaluator, error_reason
-from ldclient.impl.event_factory import _EventFactory
+from ldclient.impl.events.diagnostics import create_diagnostic_id, _DiagnosticAccumulator
+from ldclient.impl.events.event_processor import DefaultEventProcessor
+from ldclient.impl.events.types import EventFactory
 from ldclient.impl.stubs import NullEventProcessor, NullUpdateProcessor
+from ldclient.impl.util import check_uwsgi, log
 from ldclient.interfaces import BigSegmentStoreStatusProvider, FeatureRequester, FeatureStore
-from ldclient.polling import PollingUpdateProcessor
-from ldclient.streaming import StreamingUpdateProcessor
-from ldclient.util import check_uwsgi, log
 from ldclient.versioned_data_kind import FEATURES, SEGMENTS, VersionedDataKind
 from ldclient.feature_store import FeatureStore
-import queue
 
 from threading import Lock
 
 
+
 class _FeatureStoreClientWrapper(FeatureStore):
     """Provides additional behavior that the client requires before or after feature store operations.
     Currently this just means sorting the data set for init(). In the future we may also use this
@@ -61,6 +62,14 @@ class _FeatureStoreClientWrapper(FeatureStore):
         return self.store.initialized
 
 
+def _get_store_item(store, kind: VersionedDataKind, key: str) -> Any:
+    # This decorator around store.get provides backward compatibility with any custom data
+    # store implementation that might still be returning a dict, instead of our data model
+    # classes like FeatureFlag.
+    item = store.get(kind, key, lambda x: x)
+    return kind.decode(item) if isinstance(item, dict) else item
+
+
 class LDClient:
     """The LaunchDarkly SDK client object.
 
@@ -84,8 +93,8 @@ class LDClient:
 
         self._event_processor = None
         self._lock = Lock()
-        self._event_factory_default = _EventFactory(False)
-        self._event_factory_with_reasons = _EventFactory(True)
+        self._event_factory_default = EventFactory(False)
+        self._event_factory_with_reasons = EventFactory(True)
 
         store = _FeatureStoreClientWrapper(self._config.feature_store)
         self._store = store  # type: FeatureStore
@@ -94,9 +103,10 @@ class LDClient:
         self.__big_segment_store_manager = big_segment_store_manager
 
         self._evaluator = Evaluator(
-            lambda key: store.get(FEATURES, key, lambda x: x),
-            lambda key: store.get(SEGMENTS, key, lambda x: x),
-            lambda key: big_segment_store_manager.get_user_membership(key)
+            lambda key: _get_store_item(store, FEATURES, key),
+            lambda key: _get_store_item(store, SEGMENTS, key),
+            lambda key: big_segment_store_manager.get_user_membership(key),
+            log
         )
 
         if self._config.offline:
@@ -179,57 +189,62 @@ class LDClient:
     def _send_event(self, event):
         self._event_processor.send_event(event)
 
-    def track(self, event_name: str, user: dict, data: Optional[Any]=None, metric_value: Optional[AnyNum]=None):
-        """Tracks that a user performed an event.
+    def track(self, event_name: str, context: Union[dict, Context], data: Optional[Any]=None,
+              metric_value: Optional[AnyNum]=None):
+        """Tracks that an application-defined event occurred.
 
-        LaunchDarkly automatically tracks pageviews and clicks that are specified in the Goals
-        section of the dashboard. This can be used to track custom goals or other events that do
-        not currently have goals.
+        This method creates a "custom" analytics event containing the specified event name (key)
+        and context properties. You may attach arbitrary data or a metric value to the event with the
+        optional ``data`` and ``metric_value`` parameters.
 
-        :param event_name: the name of the event, which may correspond to a goal in A/B tests
-        :param user: the attributes of the user
+        Note that event delivery is asynchronous, so the event may not actually be sent until later;
+        see :func:`flush()`.
+
+        If you pass a dictionary of user attributes instead of a :class:`ldclient.Context`,
+        the SDK will convert the user to a Context. There is some overhead to this conversion,
+        so it is more efficient to pass a Context.
+
+        :param event_name: the name of the event
+        :param context: the evaluation context or user associated with the event
         :param data: optional additional data associated with the event
         :param metric_value: a numeric value used by the LaunchDarkly experimentation feature in
-          numeric custom metrics. Can be omitted if this event is used by only non-numeric metrics.
-          This field will also be returned as part of the custom event for Data Export.
+          numeric custom metrics; can be omitted if this event is used by only non-numeric metrics
         """
-        if user is None or user.get('key') is None:
-            log.warning("Missing user or user key when calling track().")
+        if not isinstance(context, Context):
+            context = Context.from_dict(context)
+        if not context.valid:
+            log.warning("Invalid context for track (%s)" % context.error)
         else:
-            self._send_event(self._event_factory_default.new_custom_event(event_name, user, data, metric_value))
+            self._send_event(self._event_factory_default.new_custom_event(event_name,
+                context, data, metric_value))
 
-    def alias(self, current_user: dict, previous_user: dict):
-        """Associates two users for analytics purposes.
+    def identify(self, context: Union[Context, dict]):
+        """Reports details about an evaluation context.
 
-        This can be helpful in the situation where a person is represented by multiple
-        LaunchDarkly users. This may happen, for example, when a person initially logs into
-        an application, the person might be represented by an anonymous user prior to logging
-        in and a different user after logging in, as denoted by a different user key.
+        This method simply creates an analytics event containing the context properties, to
+        that LaunchDarkly will know about that context if it does not already.
 
-        :param current_user: The new version of a user.
-        :param previous_user: The old version of a user.
+        Evaluating a flag, by calling :func:`variation()` or :func:`variation_detail()`, also
+        sends the context information to LaunchDarkly (if events are enabled), so you only
+        need to use :func:`identify()` if you want to identify the context without evaluating a
+        flag.
+
+        If you pass a dictionary of user attributes instead of a :class:`ldclient.Context`,
+        the SDK will convert the user to a Context. There is some overhead to this conversion,
+        so it is more efficient to pass a Context.
+
+        :param context: the context to register
         """
-        if current_user is None or current_user.get('key') is None:
-            log.warning("Missing current_user or current_user key when calling alias().")
-            return None
-        if previous_user is None or previous_user.get('key') is None:
-            log.warning("Missing previous_user or previous_user key when calling alias().")
-            return None
-        self._send_event(self._event_factory_default.new_alias_event(current_user, previous_user))
-
-    def identify(self, user: dict):
-        """Registers the user.
-
-        This simply creates an analytics event that will transmit the given user properties to
-        LaunchDarkly, so that the user will be visible on your dashboard even if you have not
-        evaluated any flags for that user. It has no other effect.
-
-        :param user: attributes of the user to register
-        """
-        if user is None or user.get('key') is None or len(str(user.get('key'))) == 0:
-            log.warning("Missing user or user key when calling identify().")
+        if not isinstance(context, Context):
+            context = Context.from_dict(context)
+        if not context.valid:
+            log.warning("Invalid context for identify (%s)" % context.error)
+        elif context.key == '' and not context.multiple:
+            # This could be a valid context for evaluations (if it was using the old user schema)
+            # but an identify event with an empty key is no good.
+            log.warning("Empty user key for identify")
         else:
-            self._send_event(self._event_factory_default.new_identify_event(user))
+            self._send_event(self._event_factory_default.new_identify_event(context))
 
     def is_offline(self) -> bool:
         """Returns true if the client is in offline mode.
@@ -258,34 +273,42 @@ class LDClient:
             return
         return self._event_processor.flush()
 
-    def variation(self, key: str, user: dict, default: Any) -> Any:
-        """Determines the variation of a feature flag for a user.
+    def variation(self, key: str, context: Union[Context, dict], default: Any) -> Any:
+        """Calculates the value of a feature flag for a given context.
+
+        If you pass a dictionary of user attributes instead of a :class:`ldclient.Context`,
+        the SDK will convert the user to a Context. There is some overhead to this conversion,
+        so it is more efficient to pass a Context.
 
         :param key: the unique key for the feature flag
-        :param user: a dictionary containing parameters for the end user requesting the flag
+        :param context: the evaluation context or user
         :param default: the default value of the flag, to be used if the value is not
           available from LaunchDarkly
-        :return: one of the flag's variation values, or the default value
+        :return: the variation for the given context, or the ``default`` value if the flag cannot be evaluated
         """
-        return self._evaluate_internal(key, user, default, self._event_factory_default).value
+        return self._evaluate_internal(key, context, default, self._event_factory_default).value
 
-    def variation_detail(self, key: str, user: dict, default: Any) -> EvaluationDetail:
-        """Determines the variation of a feature flag for a user, like :func:`variation()`, but also
-        provides additional information about how this value was calculated, in the form of an
-        :class:`ldclient.flag.EvaluationDetail` object.
+    def variation_detail(self, key: str, context: Union[Context, dict], default: Any) -> EvaluationDetail:
+        """Calculates the value of a feature flag for a given context, and returns an object that
+        describes the way the value was determined.
 
-        Calling this method also causes the "reason" data to be included in analytics events,
-        if you are capturing detailed event data for this flag.
+        The ``reason`` property in the result will also be included in analytics events, if you are
+        capturing detailed event data for this flag.
+
+        If you pass a dictionary of user attributes instead of a :class:`ldclient.Context`,
+        the SDK will convert the user to a Context. There is some overhead to this conversion,
+        so it is more efficient to pass a Context.
 
         :param key: the unique key for the feature flag
-        :param user: a dictionary containing parameters for the end user requesting the flag
+        :param context: the evaluation context or user
         :param default: the default value of the flag, to be used if the value is not
           available from LaunchDarkly
-        :return: an object describing the result
+        :return: an :class:`ldclient.evaluation.EvaluationDetail` object that includes the feature
+          flag value and evaluation reason
         """
-        return self._evaluate_internal(key, user, default, self._event_factory_with_reasons)
+        return self._evaluate_internal(key, context, default, self._event_factory_with_reasons)
 
-    def _evaluate_internal(self, key, user, default, event_factory):
+    def _evaluate_internal(self, key: str, context: Union[Context, dict], default: Any, event_factory):
         default = self._config.get_default(key, default)
 
         if self._config.offline:
@@ -298,47 +321,45 @@ class LDClient:
                 log.warning("Feature Flag evaluation attempted before client has initialized! Feature store unavailable - returning default: "
                          + str(default) + " for feature key: " + key)
                 reason = error_reason('CLIENT_NOT_READY')
-                self._send_event(event_factory.new_unknown_flag_event(key, user, default, reason))
+                self._send_event(event_factory.new_unknown_flag_event(key, context, default, reason))
                 return EvaluationDetail(default, None, reason)
 
-        if user is not None and user.get('key', "") == "":
-            log.warning("User key is blank. Flag evaluation will proceed, but the user will not be stored in LaunchDarkly.")
+        if not isinstance(context, Context):
+            context = Context.from_dict(context)
+        if not context.valid:
+            log.warning("Context was invalid for flag evaluation (%s); returning default value" % context.error)
+            return EvaluationDetail(default, None, error_reason('USER_NOT_SPECIFIED'))
 
         try:
-            flag = self._store.get(FEATURES, key, lambda x: x)
+            flag = _get_store_item(self._store, FEATURES, key)
         except Exception as e:
             log.error("Unexpected error while retrieving feature flag \"%s\": %s" % (key, repr(e)))
             log.debug(traceback.format_exc())
             reason = error_reason('EXCEPTION')
-            self._send_event(event_factory.new_unknown_flag_event(key, user, default, reason))
+            self._send_event(event_factory.new_unknown_flag_event(key, context, default, reason))
             return EvaluationDetail(default, None, reason)
         if not flag:
             reason = error_reason('FLAG_NOT_FOUND')
-            self._send_event(event_factory.new_unknown_flag_event(key, user, default, reason))
+            self._send_event(event_factory.new_unknown_flag_event(key, context, default, reason))
             return EvaluationDetail(default, None, reason)
         else:
-            if user is None or user.get('key') is None:
-                reason = error_reason('USER_NOT_SPECIFIED')
-                self._send_event(event_factory.new_default_event(flag, user, default, reason))
-                return EvaluationDetail(default, None, reason)
-
             try:
-                result = self._evaluator.evaluate(flag, user, event_factory)
+                result = self._evaluator.evaluate(flag, context, event_factory)
                 for event in result.events or []:
                     self._send_event(event)
                 detail = result.detail
                 if detail.is_default_value():
                     detail = EvaluationDetail(default, None, detail.reason)
-                self._send_event(event_factory.new_eval_event(flag, user, detail, default))
+                self._send_event(event_factory.new_eval_event(flag, context, detail, default))
                 return detail
             except Exception as e:
                 log.error("Unexpected error while evaluating feature flag \"%s\": %s" % (key, repr(e)))
                 log.debug(traceback.format_exc())
                 reason = error_reason('EXCEPTION')
-                self._send_event(event_factory.new_default_event(flag, user, default, reason))
+                self._send_event(event_factory.new_default_event(flag, context, default, reason))
                 return EvaluationDetail(default, None, reason)
 
-    def all_flags_state(self, user: dict, **kwargs) -> FeatureFlagsState:
+    def all_flags_state(self, context: Union[Context, dict], **kwargs) -> FeatureFlagsState:
         """Returns an object that encapsulates the state of all feature flags for a given user,
         including the flag values and also metadata that can be used on the front end. See the
         JavaScript SDK Reference Guide on
@@ -374,8 +395,10 @@ class LDClient:
                 log.warning("all_flags_state() called before client has finished initializing! Feature store unavailable - returning empty state")
                 return FeatureFlagsState(False)
 
-        if user is None or user.get('key') is None:
-            log.warning("User or user key is None when calling all_flags_state(). Returning empty state.")
+        if not isinstance(context, Context):
+            context = Context.from_dict(context)
+        if not context.valid:
+            log.warning("Context was invalid for all_flags_state (%s); returning default value" % context.error)
             return FeatureFlagsState(False)
 
         state = FeatureFlagsState(True)
@@ -394,14 +417,14 @@ class LDClient:
             if client_only and not flag.get('clientSide', False):
                 continue
             try:
-                detail = self._evaluator.evaluate(flag, user, self._event_factory_default).detail
+                detail = self._evaluator.evaluate(flag, context, self._event_factory_default).detail
             except Exception as e:
                 log.error("Error evaluating flag \"%s\" in all_flags_state: %s" % (key, repr(e)))
                 log.debug(traceback.format_exc())
                 reason = {'kind': 'ERROR', 'errorKind': 'EXCEPTION'}
                 detail = EvaluationDetail(None, None, reason)
 
-            requires_experiment_data = _EventFactory.is_experiment(flag, detail.reason)
+            requires_experiment_data = EventFactory.is_experiment(flag, detail.reason)
             flag_state = {
                 'key': flag['key'],
                 'value': detail.value,
@@ -417,20 +440,21 @@ class LDClient:
 
         return state
 
-    def secure_mode_hash(self, user: dict) -> str:
-        """Computes an HMAC signature of a user signed with the client's SDK key,
-        for use with the JavaScript SDK.
+    def secure_mode_hash(self, context: Union[Context, dict]) -> str:
+        """Creates a hash string that can be used by the JavaScript SDK to identify a context.
 
-        For more information, see the JavaScript SDK Reference Guide on
-        `Secure mode <https://github.com/launchdarkly/js-client#secure-mode>`_.
+        For more information, see the documentation on
+        `Secure mode <https://docs.launchdarkly.com/sdk/features/secure-mode#configuring-secure-mode-in-the-javascript-client-side-sdk>`_.
         
-        :param user: the attributes of the user
-        :return: a hash string that can be passed to the front end
+        :param context: the evaluation context or user
+        :return: the hash string
         """
-        key = user.get('key')
-        if key is None or self._config.sdk_key is None:
+        if not isinstance(context, Context):
+            context = Context.from_dict(context)
+        if not context.valid:
+            log.warning("Context was invalid for secure_mode_hash (%s); returning empty hash" % context.error)
             return ""
-        return hmac.new(self._config.sdk_key.encode(), key.encode(), hashlib.sha256).hexdigest()
+        return hmac.new(str(self._config.sdk_key).encode(), context.fully_qualified_key.encode(), hashlib.sha256).hexdigest()
 
     @property
     def big_segment_store_status_provider(self) -> BigSegmentStoreStatusProvider:
