@@ -7,7 +7,7 @@ Note that the same class can also be imported from the ``ldclient.client`` submo
 from typing import Optional, Callable, List, Set
 
 from ldclient.feature_store import InMemoryFeatureStore
-from ldclient.impl.util import log
+from ldclient.impl.util import log, validate_application_info
 from ldclient.interfaces import BigSegmentStore, EventProcessor, FeatureStore, UpdateProcessor
 
 GET_LATEST_FEATURES_PATH = '/sdk/latest-flags'
@@ -60,15 +60,15 @@ class BigSegmentsConfig:
         self.__status_poll_interval = status_poll_interval
         self.__stale_after = stale_after
         pass
-    
+
     @property
     def store(self) -> Optional[BigSegmentStore]:
         return self.__store
-    
+
     @property
     def context_cache_size(self) -> int:
         return self.__context_cache_size
-    
+
     @property
     def context_cache_time(self) -> float:
         return self.__context_cache_time
@@ -77,7 +77,7 @@ class BigSegmentsConfig:
     def user_cache_size(self) -> int:
         """Deprecated alias for :attr:`context_cache_size`."""
         return self.context_cache_size
-    
+
     @property
     def user_cache_time(self) -> float:
         """Deprecated alias for :attr:`context_cache_time`."""
@@ -86,7 +86,7 @@ class BigSegmentsConfig:
     @property
     def status_poll_interval(self) -> float:
         return self.__status_poll_interval
-    
+
     @property
     def stale_after(self) -> float:
         return self.__stale_after
@@ -169,7 +169,7 @@ class Config:
                  initial_reconnect_delay: float=1,
                  defaults: dict={},
                  send_events: Optional[bool]=None,
-                 update_processor_class: Optional[Callable[[str, 'Config', FeatureStore], UpdateProcessor]]=None, 
+                 update_processor_class: Optional[Callable[[str, 'Config', FeatureStore], UpdateProcessor]]=None,
                  poll_interval: float=30,
                  use_ldd: bool=False,
                  feature_store: Optional[FeatureStore]=None,
@@ -188,7 +188,8 @@ class Config:
                  wrapper_name: Optional[str]=None,
                  wrapper_version: Optional[str]=None,
                  http: HTTPConfig=HTTPConfig(),
-                 big_segments: Optional[BigSegmentsConfig]=None):
+                 big_segments: Optional[BigSegmentsConfig]=None,
+                 application: Optional[dict]=None):
         """
         :param sdk_key: The SDK key for your LaunchDarkly account. This is always required.
         :param base_uri: The base URL for the LaunchDarkly server. Most users should use the default
@@ -256,6 +257,7 @@ class Config:
           servers.
         :param http: Optional properties for customizing the client's HTTP/HTTPS behavior. See
           :class:`HTTPConfig`.
+        :param application: Optional properties for setting application metadata. See :py:attr:`~application`
         """
         self.__sdk_key = sdk_key
 
@@ -287,6 +289,7 @@ class Config:
         self.__wrapper_version = wrapper_version
         self.__http = http
         self.__big_segments = BigSegmentsConfig() if not big_segments else big_segments
+        self.__application = validate_application_info(application or {}, log)
 
     def copy_with_new_sdk_key(self, new_sdk_key: str) -> 'Config':
         """Returns a new ``Config`` instance that is the same as this one, except for having a different SDK key.
@@ -459,9 +462,21 @@ class Config:
     def big_segments(self) -> BigSegmentsConfig:
         return self.__big_segments
 
+    @property
+    def application(self) -> dict:
+        """
+        An object that allows configuration of application metadata.
+
+        Application metadata may be used in LaunchDarkly analytics or other
+        product features, but does not affect feature flag evaluations.
+
+        If you want to set non-default values for any of these fields, provide
+        the appropriately configured dict to the {Config} object.
+        """
+        return self.__application
+
     def _validate(self):
         if self.offline is False and self.sdk_key is None or self.sdk_key == '':
             log.warning("Missing or blank sdk_key.")
 
-
 __all__ = ['Config', 'BigSegmentsConfig', 'HTTPConfig']
