@@ -9,6 +9,12 @@ from ldclient.interfaces import DiagnosticDescription, FeatureStore, FeatureStor
 from ldclient.versioned_data_kind import VersionedDataKind
 from ldclient.feature_store import CacheConfig
 
+def _ensure_encoded(kind, item):
+    return item if isinstance(item, dict) else kind.encode(item)
+
+def _is_deleted(item):
+    return item is not None and item.get('deleted') is True
+
 class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
     """A partial implementation of :class:`ldclient.interfaces.FeatureStore`.
 
@@ -32,16 +38,20 @@ class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
             self._cache = None
         self._inited = False
 
-    def init(self, all_data: Mapping[VersionedDataKind, Mapping[str, Dict[Any, Any]]]):
+    def init(self, all_encoded_data: Mapping[VersionedDataKind, Mapping[str, Dict[Any, Any]]]):
         """
         """
-        self._core.init_internal(all_data)
+        self._core.init_internal(all_encoded_data)  # currently FeatureStoreCore expects to receive dicts
         if self._cache is not None:
             self._cache.clear()
-            for kind, items in all_data.items():
-                self._cache[self._all_cache_key(kind)] = self._items_if_not_deleted(items)
+            for kind, items in all_encoded_data.items():
+                decoded_items = {}  # we don't want to cache dicts, we want to cache FeatureFlags/Segments
                 for key, item in items.items():
-                    self._cache[self._item_cache_key(kind, key)] = [item] # note array wrapper
+                    decoded_item = kind.decode(item)
+                    self._cache[self._item_cache_key(kind, key)] = [decoded_item] # note array wrapper
+                    if not _is_deleted(decoded_item):
+                        decoded_items[key] = decoded_item
+                self._cache[self._all_cache_key(kind)] = decoded_items
         self._inited = True
 
     def get(self, kind, key, callback=lambda x: x):
@@ -52,11 +62,13 @@ class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
             cached_item = self._cache.get(cache_key)
             # note, cached items are wrapped in an array so we can cache None values
             if cached_item is not None:
-                return callback(self._item_if_not_deleted(cached_item[0]))
-        item = self._core.get_internal(kind, key)
+                item = cached_item[0]
+                return callback(None if _is_deleted(item) else item)
+        encoded_item = self._core.get_internal(kind, key)  # currently FeatureStoreCore returns dicts
+        item = None if encoded_item is None else kind.decode(encoded_item)
         if self._cache is not None:
             self._cache[cache_key] = [item]
-        return callback(self._item_if_not_deleted(item))
+        return callback(None if _is_deleted(item) else item)
 
     def all(self, kind, callback=lambda x: x):
         """
@@ -66,7 +78,12 @@ class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
             cached_items = self._cache.get(cache_key)
             if cached_items is not None:
                 return callback(cached_items)
-        items = self._items_if_not_deleted(self._core.get_all_internal(kind))
+        encoded_items = self._core.get_all_internal(kind)
+        all_items = {}
+        if encoded_items is not None:
+            for key, item in encoded_items.items():
+                all_items[key] = kind.decode(item)
+        items = self._items_if_not_deleted(all_items)
         if self._cache is not None:
             self._cache[cache_key] = items
         return callback(items)
@@ -77,12 +94,14 @@ class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
         deleted_item = { "key": key, "version": version, "deleted": True }
         self.upsert(kind, deleted_item)
 
-    def upsert(self, kind, item):
+    def upsert(self, kind, encoded_item):
         """
         """
-        new_state = self._core.upsert_internal(kind, item)
+        encoded_item = _ensure_encoded(kind, encoded_item)
+        new_state = self._core.upsert_internal(kind, encoded_item)
+        new_decoded_item = kind.decode(new_state)
         if self._cache is not None:
-            self._cache[self._item_cache_key(kind, item.get('key'))] = [new_state]
+            self._cache[self._item_cache_key(kind, new_decoded_item.get('key'))] = [new_decoded_item]
             self._cache.pop(self._all_cache_key(kind), None)
 
     @property
@@ -115,12 +134,6 @@ class CachingStoreWrapper(DiagnosticDescription, FeatureStore):
     def _all_cache_key(kind):
         return kind.namespace
     
-    @staticmethod
-    def _item_if_not_deleted(item):
-        if item is not None and item.get('deleted', False):
-            return None
-        return item
-    
     @staticmethod
     def _items_if_not_deleted(items):
         results = {}
