@@ -1,22 +1,19 @@
-import pytest
-import json
 import time
-from ldclient.client import LDClient, Config
+from ldclient.client import LDClient, Config, Context
 from ldclient.config import BigSegmentsConfig
-from ldclient.evaluation import BigSegmentsStatus
+from ldclient.evaluation import BigSegmentsStatus, EvaluationDetail
 from ldclient.feature_store import InMemoryFeatureStore
-from ldclient.flag import EvaluationDetail
 from ldclient.impl.big_segments import _hash_for_user_key
 from ldclient.impl.evaluator import _make_big_segment_ref
 from ldclient.interfaces import FeatureStore
 from ldclient.versioned_data_kind import FEATURES, SEGMENTS
-from testing.impl.evaluator_util import make_boolean_flag_matching_segment
+
+from testing.builders import *
 from testing.mock_components import MockBigSegmentStore
 from testing.stub_util import MockEventProcessor, MockUpdateProcessor
-from testing.test_ldclient import make_off_flag_with_value
+from testing.test_ldclient import make_client, user
 
 
-user = { 'key': 'userkey' }
 flag1 = {
     'key': 'key1',
     'version': 100,
@@ -55,13 +52,6 @@ class ErroringFeatureStore(FeatureStore):
     def initialized(self):
         return True
 
-def make_client(store):
-    return LDClient(config=Config(sdk_key='SDK_KEY',
-                                  base_uri='http://test',
-                                  event_processor_class=MockEventProcessor,
-                                  update_processor_class=MockUpdateProcessor,
-                                  feature_store=store))
-
 def get_log_lines(caplog, level):
     loglines = caplog.records
     if callable(loglines):
@@ -71,44 +61,64 @@ def get_log_lines(caplog, level):
 
 
 def test_variation_for_existing_feature():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
     assert 'value' == client.variation('feature.key', user, default='default')
 
+def test_variation_passes_user_to_evaluator():
+    u = {'key': 'userkey'}
+    feature =  FlagBuilder('feature.key').on(True).variations('wrong', 'right').target(1, 'userkey').build()
+    store = InMemoryFeatureStore()
+    store.init({FEATURES: {'feature.key': feature}})
+    client = make_client(store)
+    assert 'right' == client.variation('feature.key', u, default='default')
+
+def test_variation_passes_context_to_evaluator():
+    c = Context.create('userkey')
+    feature =  FlagBuilder('feature.key').on(True).variations('wrong', 'right').target(1, 'userkey').build()
+    store = InMemoryFeatureStore()
+    store.init({FEATURES: {'feature.key': feature}})
+    client = make_client(store)
+    assert 'right' == client.variation('feature.key', c, default='default')
+
 def test_variation_for_unknown_feature():
     store = InMemoryFeatureStore()
     client = make_client(store)
     assert 'default' == client.variation('feature.key', user, default='default')
 
 def test_variation_when_user_is_none():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
     assert 'default' == client.variation('feature.key', None, default='default')
 
 def test_variation_when_user_has_no_key():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
     assert 'default' == client.variation('feature.key', { }, default='default')
 
+def test_variation_for_invalid_context():
+    c = Context.create('')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
+    store = InMemoryFeatureStore()
+    store.init({FEATURES: {'feature.key': feature}})
+    client = make_client(store)
+    assert 'default' == client.variation('feature.key', c, default='default')
+
 def test_variation_for_flag_that_evaluates_to_none():
-    empty_flag = {
-        'key': 'feature.key',
-        'on': False,
-        'offVariation': None
-    }
+    empty_flag = FlagBuilder('feature.key').on(False).build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': empty_flag}})
     client = make_client(store)
     assert 'default' == client.variation('feature.key', user, default='default')
 
 def test_variation_detail_for_existing_feature():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
@@ -122,7 +132,7 @@ def test_variation_detail_for_unknown_feature():
     assert expected == client.variation_detail('feature.key', user, default='default')
 
 def test_variation_detail_when_user_is_none():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
@@ -130,7 +140,7 @@ def test_variation_detail_when_user_is_none():
     assert expected == client.variation_detail('feature.key', None, default='default')
 
 def test_variation_detail_when_user_has_no_key():
-    feature = make_off_flag_with_value('feature.key', 'value')
+    feature = build_off_flag_with_value('feature.key', 'value').build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': feature}})
     client = make_client(store)
@@ -138,11 +148,7 @@ def test_variation_detail_when_user_has_no_key():
     assert expected == client.variation_detail('feature.key', { }, default='default')
 
 def test_variation_detail_for_flag_that_evaluates_to_none():
-    empty_flag = {
-        'key': 'feature.key',
-        'on': False,
-        'offVariation': None
-    }
+    empty_flag = FlagBuilder('feature.key').on(False).build()
     store = InMemoryFeatureStore()
     store.init({FEATURES: {'feature.key': empty_flag}})
     client = make_client(store)
@@ -169,12 +175,7 @@ def test_variation_detail_when_feature_store_throws_error(caplog):
     assert errlog == [ 'Unexpected error while retrieving feature flag "feature.key": NotImplementedError()' ]
 
 def test_flag_using_big_segment():
-    segment = {
-        'key': 'segkey',
-        'version': 1,
-        'generation': 1,
-        'unbounded': True
-    }
+    segment = SegmentBuilder('segkey').unbounded(True).generation(1).build()
     flag = make_boolean_flag_matching_segment(segment)
     store = InMemoryFeatureStore()
     store.init({ FEATURES: { flag['key']: flag }, SEGMENTS: { segment['key']: segment } })
